#ifndef __XTEXT_H__
#define __XTEXT_H__

#include <gdk/gdk.h>
#include <gtk/gtkadjustment.h>
#include <gtk/gtkwidget.h>

#include <X11/Xlib.h>
#include <X11/Xatom.h>


/*#define GTK_XTEXT(obj)          GTK_CHECK_CAST (obj, gtk_xtext_get_type (), GtkXText)*/
#define GTK_XTEXT(obj) ((GtkXText*)obj)
#define GTK_XTEXT_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gtk_xtext_get_type (), GtkXTextClass)
#define GTK_IS_XTEXT(obj)       GTK_CHECK_TYPE (obj, gtk_xtext_get_type ())

#define FONT_1BYTE 0
#define FONT_2BYTE 1
#define FONT_SET 2

#define ATTR_BOLD '\002'
#define ATTR_COLOR '\003'
#define ATTR_BEEP '\007'
#define ATTR_RESET '\017'
#define ATTR_REVERSE '\026'
#define ATTR_ESCAPE '\033'
#define ATTR_UNDERLINE '\037'

typedef struct _GtkXText         GtkXText;
typedef struct _GtkXTextClass    GtkXTextClass;

typedef struct textentry
{
   struct textentry *next;
   char *str;
   int str_len;
   int str_width;
   int mark_start;
   int mark_end;
} textentry;

struct _GtkXText
{
   GtkWidget widget;

   GtkAdjustment *adj;
   gfloat old_value;    /* last known adj->value */
   GdkPixmap *pixmap;   /* 0 = use palette[1] */

   GdkGC *bgc;          /* backing pixmap */
   GdkGC *fgc;          /* text foreground color */
   gulong palette[18];

   textentry *text_first;
   textentry *text_last;

   gint io_tag;         /* for delayed refresh events */

   XFontStruct *xfont;
   GdkFont *font;
   int fontsize;
   int fonttype;
   int fontwidth[256];  /* each char's width, only for FONT_1BYTE type */
   int space_width;     /* width (pixels) of the space " " character */

   int indent;          /* position of separator (pixels) from left */

   int select_start_adj;   /* the adj->value when the selection started */
   int select_start_x;
   int select_start_y;
   int select_end_x;
   int select_end_y;

   int num_lines;
   int max_lines;

   int last_subline;
   int last_line;
   textentry *last_ent;

   int col_fore;
   int col_back;

   int frozen;

   char num[8];   /* for parsing mirc color */
   int nc;        /* offset into xtext->num */

   unsigned int parsing_backcolor:1;
   unsigned int parsing_color:1;
   unsigned int backcolor:1;
   unsigned int button_down:1;
   unsigned int bold:1;
   unsigned int underline:1;
   unsigned int reverse:1;
   unsigned int transparent:1;
   unsigned int separator:1;
   unsigned int shaded:1;
};

struct _GtkXTextClass
{
   GtkWidgetClass parent_class;
};

GtkWidget* gtk_xtext_new         (int indent, int separator);
guint      gtk_xtext_get_type    (void);
void       gtk_xtext_append      (GtkXText *xtext, char *text, int len);
void       gtk_xtext_append_indent (GtkXText *xtext,
                                    char *left_text, int left_len,
                                    char *right_text, int right_len);
void       gtk_xtext_set_font    (GtkXText *xtext, GdkFont *font, char *name);

void       gtk_xtext_set_background (GtkXText *xtext, GdkPixmap *pixmap, int shaded);
void       gtk_xtext_set_palette (GtkXText *xtext, GdkColor palette[]);
void       gtk_xtext_remove_lines (GtkXText *xtext, int lines, int refresh);
void       gtk_xtext_refresh     (GtkXText *xtext);
void       gtk_xtext_thaw        (GtkXText *xtext);
void       gtk_xtext_freeze      (GtkXText *xtext);
gchar*     gtk_xtext_get_chars(GtkXText *xtext);
#endif

